import abc
from ec2init.execution.base_async_procedure import BaseAsyncProcedure
from ec2init.execution.procedure import Procedure

from concurrent.futures import ThreadPoolExecutor, Executor
from typing import List, Any
import threading
import logging

logger = logging.getLogger(__name__)

class MultiThreadProcedure(BaseAsyncProcedure):

    def create_pool(self) -> ThreadPoolExecutor:
        return ThreadPoolExecutor(max_workers=self.max_workers, thread_name_prefix=self.name)
    
    def submit_task(self, pool :Executor, task :Any):
        return pool.submit(self.task_function, task)

    def post_pool_shutdown(self) -> None:
        pass 
    
    def is_running(self) -> bool:
        return self.is_already_running(self.name)

    @staticmethod
    def is_already_running(name :str) -> bool:
        running_threads = threading.enumerate()
        for t in running_threads:
            if t.name.startswith(name):
                return True
        return False

    @abc.abstractmethod
    def task_function(self, task :Any) -> None:
        pass