import abc
from configparser import ConfigParser

from typing import Sequence

from ec2init.execution.procedure import Procedure

class Plan(abc.ABC):
    """
    Plan to be executed by the Executor.
    """

    @abc.abstractmethod
    def init(self, name: str, config: ConfigParser, **kwargs: str) -> None:
        """
        Initialization method
        """
        pass

    @abc.abstractmethod
    def create(self) -> Sequence[Procedure]:
        """
        Method to create a Plan. Depending on the current
        state, different Procedures can be executed on
        each run.
        """
        pass

    @abc.abstractmethod
    def report_error(self, error_message: str) -> None:
        """
        Method to report a final non-recoverable error to
        Hopsworks Cloud
        """
        pass

    @abc.abstractmethod
    def report_configuration_error(self, error_message: str) -> None:
        """
        Method to report a user configuration error to
        Hopsworks Cloud
        """
        pass