import logging
from pathlib import Path
from ec2init.utils.bucket_configuration import BucketConfiguration
from typing import List
from ec2init import Procedure
from ec2init.utils.hopsfs import HopsFSUtils

logger = logging.getLogger(__name__)

class FixHdfsSiteForAzure(Procedure):
    def __init__(self, name: str, check_file: Path, container_configuration: BucketConfiguration, storage_account: str, client_id:str) -> None:
        self.name = name
        self.check_file = check_file
        self.container_configuration = container_configuration
        self.storage_account = storage_account
        self.client_id = client_id

    def pre(self) -> None:
        pass

    def fix_hdfs_site_for_azure(self, storage_account: str, container:str, client_id:str) -> None:
        HopsFSUtils.fix_hdfs_site("dfs.replication", "1")
        HopsFSUtils.fix_hdfs_site("dfs.enable.cloud.persistence", "true")
        HopsFSUtils.fix_hdfs_site("dfs.cloud.provider", "azure")
        HopsFSUtils.fix_hdfs_site("dfs.cloud.azure.container", container)
        HopsFSUtils.fix_hdfs_site("dfs.cloud.azure.storage", storage_account)
        HopsFSUtils.fix_hdfs_site("dfs.disable.non.cloud.storage.policies", "true"); 
        if client_id:
            HopsFSUtils.fix_hdfs_site("dfs.azure.mgm.identity.client.id", client_id)

    def execute(self) -> List[Procedure]:
        try :
            #add azure related properties to hdfs-site.xml
            self.friendly_name = "Unable to set Azure related properties in hdfs-site.xml"
            self.fix_hdfs_site_for_azure(self.storage_account, self.container_configuration.name, self.client_id)

            #Every thing is OK. 
            if self.check_file is not None:
                self.check_file.touch()
        except Exception as ex:
            logger.warn("%s - Could not update hdfs-site params for azure. Container: %s %s", self.name, self.container_configuration.name, ex)
            raise Exception(self.friendly_name) 

        return []

    def rollback(self) -> None:
        pass