import logging
from pathlib import Path
from ec2init.utils.cloud import get_cloud_provider
from ec2init.utils.constants import Scripts
from ec2init.utils.glassfish_asadmin import AsadminUtils

from configparser import ConfigParser
from typing import List

from ec2init import Procedure
from ec2init.utils.system import SystemUtils

logger = logging.getLogger(__name__)

class ConfigureDASNode(Procedure):
    def __init__(self, name: str, config: ConfigParser, check_file: Path, has_configured_das_node: bool, cloud_provider: str, head_node_ips: List[str], version: str) -> None:
        self.name = name
        self.config = config
        self.cloud_provider = cloud_provider
        self.head_node_ips = head_node_ips
        self.version = version
        self.check_file = check_file
        self.has_configured_das_node = has_configured_das_node

    def pre(self) -> None:
        if self.report_callback is not None:
            try:
                self.report_callback.execute()
            except Exception as ex:
                logger.warning("%s - Could not execute pre-execute: %s", self.name, ex)

    def execute(self) -> List[Procedure]:
        if not self.has_configured_das_node:
            private_ip = get_cloud_provider(self.cloud_provider).get_private_ip(self.config)
            data_dir = Path(self.config.get('initplan', 'hops_data_dir', fallback='/srv/hops/hopsdata'))
            kagent_conf_dir = self.config.get('general', 'kagent_conf_dir', fallback="/srv/hops/kagent/etc")
            services_file = Path(kagent_conf_dir, "services")
            try:
                AsadminUtils.configure_das_node(private_ip, self.head_node_ips, self.version, data_dir, services_file)            
            except Exception as e:
                self.friendly_name = str(e)
                raise Exception(self.friendly_name)

            output = SystemUtils.execute_command([Scripts.MYSQL_CLIENT, "hopsworks", "-e", f"REPLACE INTO variables (id, value, visibility) VALUES ('ha_enabled', 'true', '1')"])
            if output.returncode > 0:
              self.friendly_name = output.stdout.decode("utf-8")
              raise Exception(self.friendly_name)

        if self.check_file:
            self.check_file.touch()

        return []

    def rollback(self) -> None:
        pass