import logging

from typing import List

from ec2init import Procedure
from pathlib import Path
import docker

logger = logging.getLogger(__name__)

class LoadDocker(Procedure):
    def __init__(self, name: str, check_file: Path, default_registry_url: str, python_image_name: str, hopsworks_version: str) -> None:
        self.name = name
        self.check_file = check_file
        self.image = f"{default_registry_url}/{python_image_name}:{hopsworks_version}" 

    def pre(self) -> None:
        pass

    def execute(self) -> List[Procedure]:
        docker_client = docker.from_env(timeout=120)
        try :
            LoadDocker.load_docker_image(docker_client, self.image, detach=True)
            #Every thing is OK. 
            if self.check_file is not None:
                self.check_file.touch()
            logger.info("%s - loading docker image (%s)", self.name, self.image)
        except Exception as ex:
            logger.warning("%s - Exception %s", self.name, ex)
            raise ex

        return []

    @staticmethod
    def load_docker_image(docker_client, image, detach=True):
        return docker_client.containers.run(image, 'sh -c "find . -type f -print0 | xargs -I {} -0 -P 4 dd if={} of=/dev/null bs=1M"', detach=detach, auto_remove=True)

    def rollback(self) -> None:
        pass
