import pathlib
import logging
from typing import List
from ec2init import Procedure
from pathlib import Path
from configparser import ConfigParser
from ec2init.utils.system import SystemUtils

logger = logging.getLogger(__name__)

class UserInitScript(Procedure):
    def __init__(self, name: str, config: ConfigParser, check_file: Path, user_init_script: str) -> None:
        self.name = name
        self.config = config
        self.check_file = check_file
        self.user_init_script = user_init_script

    def execute(self) -> List[Procedure]:
        try:
            # run user init script
            user_init_script_path = pathlib.Path(self.user_init_script)
            if user_init_script_path.is_file():
                SystemUtils.execute_command(["bash", "-c", self.user_init_script], timeout=None)
            
            # Every thing is OK.
            if self.check_file is not None:
                self.check_file.touch()
        except Exception as ex:
            logger.warning("%s - Could not initialize cluster.", ex)
            raise ex

        return []

    def rollback(self) -> None:
        pass
