import logging
from configparser import ConfigParser
from pathlib import Path

from typing import List

from ec2init import Procedure
from ec2init.utils.cloud import get_cloud_provider
from ec2init.utils.offline_backup_information import OffLineBackupInformation

logger = logging.getLogger(__name__)

class StoreBackupInfo(Procedure):
    def __init__(self, name: str, config: ConfigParser, file: Path, check_file: Path, cloud_provider: str, user_registry_url: str, search_domain: str, node_id: str) -> None:
        self.name = name
        self.config = config
        self.file = file
        self.check_file = check_file
        self.cloud_provider = cloud_provider
        self.user_registry_url = user_registry_url
        self.search_domain = search_domain
        self.node_id = node_id

    def pre(self) -> None:
        pass
    
    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)
        try:
            if self.file.is_file():
                self.file.unlink()
            fqdn = get_cloud_provider(self.cloud_provider).get_fqdn(self.config, self.search_domain, self.node_id)
            local_ip = get_cloud_provider(self.cloud_provider).get_private_ip(self.config)
            offline_backup_info = OffLineBackupInformation(self.config)
            offline_backup_info.ip = local_ip
            offline_backup_info.hostname = fqdn
            offline_backup_info.user_registry_url = self.user_registry_url
            offline_backup_info.save(self.file)
            
            # Every thing is OK.
            if self.check_file is not None:
                self.check_file.touch()
        
        except Exception as ex:
            logger.warning("%s - Could not store backup info.", ex)
            raise ex

        logger.info("%s - Finished my task!", self.name)
        return []

        
    def rollback(self) -> None:
        pass