import logging
import configparser

from typing import Sequence, List

from ec2init import Procedure
from ec2init.plans.init.base_init_plan import BaseInitPlan

logger = logging.getLogger(__name__)

class UserInitPlan(BaseInitPlan):
    def init(self, name: str, config: configparser.ConfigParser, **kwargs: str) -> None:
        super().init(name, config, **kwargs)

    def create(self) -> Sequence[Procedure]:
        plan: List[Procedure] = []

        # do not call back hopsworks in case the user is setting a proxy. 
        # /etc/environment will not have been sourced at this point
        self.add_user_init_script(plan, call_back=False) 
        
        parentPlan: Sequence[Procedure] = super().create()
        plan += list(parentPlan)
        return plan

class MasterUserInitPlan(UserInitPlan):
    def init(self, name: str, config: configparser.ConfigParser, **kwargs: str) -> None:
        super().init(name, config, **kwargs)
        self.instance_type = "master"

class WorkerUserInitPlan(UserInitPlan):
    def init(self, name: str, config: configparser.ConfigParser, **kwargs: str) -> None:
        super().init(name, config, **kwargs)
        self.instance_type = "worker"

class RonDBAllInOneUserInitPlan(UserInitPlan):
    def init(self, name: str, config: configparser.ConfigParser, **kwargs: str) -> None:
        super().init(name, config, **kwargs)
        self.instance_type = "rondb_all-in-one"

class MgmRonDBUserInitPlan(UserInitPlan):
    def init(self, name: str, config: configparser.ConfigParser, **kwargs: str) -> None:
        super().init(name, config, **kwargs)
        self.instance_type = "ndb_mgm"

class NdbdRonDBUserInitPlan(UserInitPlan):
    def init(self, name: str, config: configparser.ConfigParser, **kwargs: str) -> None:
        super().init(name, config, **kwargs)
        self.instance_type = "ndb_ndbd"

class MysqldRonDBUserInitPlan(UserInitPlan):
    def init(self, name: str, config: configparser.ConfigParser, **kwargs: str) -> None:
        super().init(name, config, **kwargs)
        self.instance_type = "mysqld"

class ApiRonDBUserInitPlan(UserInitPlan):
    def init(self, name: str, config: configparser.ConfigParser, **kwargs: str) -> None:
        super().init(name, config, **kwargs)
        self.instance_type = "ndb_api"