from typing import Dict, List
from cryptography import x509
from cryptography.x509 import ObjectIdentifier
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives import hashes


def generate_private_key(key_size: int = 2048) -> rsa.RSAPrivateKeyWithSerialization:
    return rsa.generate_private_key(
        public_exponent=65537,
        key_size=key_size,
        backend=default_backend())

def serialize_private_key_rsa(key: rsa.RSAPrivateKeyWithSerialization) -> bytes:
    return key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption())

def generate_certificate_signing_request(key: rsa.RSAPrivateKeyWithSerialization, names: Dict[ObjectIdentifier, str]) -> x509.CertificateSigningRequest :
    nameAttrs: List[x509.NameAttribute] = []
    for n, v in names.items():
        nameAttrs.append(x509.NameAttribute(n, v))
    
    return x509.CertificateSigningRequestBuilder() \
        .subject_name(x509.Name(nameAttrs)) \
        .sign(key, hashes.SHA256())

def serialize_certificate_signing_request(csr: x509.CertificateSigningRequest) -> bytes:
    return csr.public_bytes(serialization.Encoding.PEM)