from enum import Enum
from operator import contains
from typing import List 

class Service(Enum):
    Kagent = "kagent"
    Consul = "consul"
    Namenode = "namenode"
    Datanode = "datanode"
    MySQLd = "mysqld"
    Opensearch = "opensearch"
    Resourcemanager = "resourcemanager"
    Zookeeper = "zookeeper"
    Kafka = "kafka"
    Onlinefs = "onlinefs"
    Hivemetastore = "hivemetastore"
    Hiveserver2 = "hiveserver2"
    Logstash = "logstash"
    GlassfishInstance = "glassfish-instance"
    
    # not set by the user
    ElasticExporter = "elastic_exporter"
    MySQLdExporter = "mysqld_exporter"
    NodeExporter = "node_exporter"
    FilebeatService = "filebeat-service"
    Docker = "docker"

    # only on master (primary head node)
    Alertmanager = "alertmanager"
    CloudMetricsCollector = "cloud-metrics-collector"
    Glassfish = "glassfish-domain1"
    Grafana = "grafana"
    Pushgateway = "pushgateway"
    Prometheus = "prometheus"
    AirflowWebserver = "airflow-webserver"
    AirflowScheduler = "airflow-scheduler"
    ePipe = "epipe"
    Historyserver = "historyserver"
    FlinkHistoryserver = "flinkhistoryserver"
    SparkHistoryserver = "sparkhistoryserver"
    OpensearchDashboards = "opensearch-dashboards"
    Livy = "livy"
    FilebeatJupyter = "filebeat-jupyter"
    FilebeatTFServing = "filebeat-tf-serving"
    FilebeatSklearnServing = "filebeat-sklearn-serving"

    # only on workers 
    Nodemanager = "nodemanager"
    FilebeatSpark = "filebeat-spark"

    # only mysqld node
    FlyingDuck = "flyingduck"

class ServiceUtils:

    @staticmethod
    def stringCSToServiceList(input : str) -> List[Service]:
        return [Service(e) for e in input.split(",") if e.strip() != ""]

    @staticmethod
    def contains(services : List[Service], service : str) -> bool:
        return len([e for e in services if e.value == service]) != 0 

    @staticmethod
    def containsKey(services : List[Service], service : Service) -> bool:
        return len([e for e in services if e == service]) != 0 

    @staticmethod
    def containsValue(services : List[str], service : Service) -> bool:
        return len([e for e in services if e == service.value]) != 0 

    @staticmethod
    def getServicesList(input : str) -> List[Service]:
        services_list = ServiceUtils.stringCSToServiceList(input)
        if len(services_list) == 0:
            return services_list
            
        if not ServiceUtils.containsKey(services_list, Service.ElasticExporter) and ServiceUtils.containsKey(services_list, Service.Opensearch):
            services_list.append(Service.ElasticExporter)
        
        if not ServiceUtils.containsKey(services_list, Service.MySQLdExporter) and ServiceUtils.containsKey(services_list, Service.MySQLd):
            services_list.append(Service.MySQLdExporter)

        if not ServiceUtils.containsKey(services_list, Service.NodeExporter):
            services_list.append(Service.NodeExporter)
        
        if not ServiceUtils.containsKey(services_list, Service.FilebeatService):
            services_list.append(Service.FilebeatService)
        
        if not ServiceUtils.containsKey(services_list, Service.Docker):
            services_list.append(Service.Docker)

        return services_list
